#!/usr/bin/python

# Klas Jareteg, 2012-10-14
# Description:
#   Analyzing comparison case for coupled and segregated solvers

import sys, os, stat
import logging
from optparse import OptionParser
import re
import ConfigParser
import math
import numpy as np
import matplotlib.pyplot as plt
import shutil

def dualAxisPlotComparison(y1,y2,x,data,**kwargs):
    fig = plt.figure()
    ax1 = fig.add_subplot(1,1,1)
    plt.xlabel(x['label'])
    plt.ylabel(y1['label'])
    plt.grid()
   
    ls = ['-','--','-.',':']
    if 'LineStyle' in kwargs:
        ls = kwargs['LineStyle']
        
    if 'log' in y1:
        if y1['log']:
            ax1.set_yscale('log')
    
    for i in range(len(data)):
        d = data[i]
        X = d[x['fields'][0]]['data']
        subsety1=(0,len(X))
        if 'xlimit' in kwargs:
            maxx = kwargs['xlimit']
            ii = len(X)
            for j in range(len(X)):
                if X[j]>maxx:
                    ii = j
                    break
            subsety1 = (0,ii)
        
        for f,c in zip(y1['fields'],y1['colors']):
            Y = d[f]['data'][subsety1[0]:subsety1[1]]
            Xs = X[subsety1[0]:subsety1[1]]
            if 'Single' in kwargs:
                if 'titles' in y1:
                    line, = ax1.plot(Xs,Y,color=y1['colors'][i],label=y1['titles'][i],ls=ls[i])
                else:
                    line, = ax1.plot(Xs,Y,color=y1['colors'][i],label=d[f]['name'],ls=ls[i])
            else:
                if i==0:
                    line, = ax1.plot(Xs,Y,color=c,label=d[f]['name'],ls=ls[i])
                else:
                    line, = ax1.plot(Xs,Y,color=c,ls=ls[i])

    if y2: 
        ax2 = plt.twinx() 
        if 'log' in y2:
            if y2['log']:
                ax2.set_yscale('log')
    
        plt.ylabel(y2['label'])
        for i in range(len(data)):
            d = data[i]
            X = d[x['fields'][0]]['data']
            subsety2=(0,len(X))
            if 'xlimit' in kwargs:
                maxx = kwargs['xlimit']
                ii = len(X)
                for j in range(len(X)):
                    if X[j]>maxx:
                        ii = j
                        break
                subsety2 = (0,ii)

            for f,c in zip(y2['fields'],y2['colors']):
                Y = d[f]['data'][subsety2[0]:subsety2[1]]
                Xs = X[subsety2[0]:subsety2[1]]
                line, = ax2.plot(Xs,Y,color=c, ls=ls[i])
     
    handles, labels = ax1.get_legend_handles_labels()
    ax1.legend(handles[::-1], labels[::-1])

    if 'title' in kwargs:
        ax1.set_title(kwargs['title'])

    if 'name' in kwargs:
        plt.savefig('%s.pdf'%kwargs['name'],format='pdf')
    else:
        plt.savefig('Output.pdf',format='pdf')
    
def segregatedResiduals(log, dim, pSolver, USolver):
    t = []
    i = []
    e = []
    p = []
    U = []
    Ux = []
    Uy = []
    Uz = []
    k = []
    eps = []
    
    f = open(log)
    started = False
    ux = 0
    for line in f:
        ls = re.sub(r'\s+',' ',line.strip()).rsplit()
        if len(ls)>0:
            if started:
                if ls[0]=="Time":
                    t.append(float(ls[2]))
                elif ls[0]=="%s:"%pSolver:
                        p.append(float(ls[7][0:-1]))
                        i.append(int(ls[-1]))         
                elif ls[0]=="%s:"%USolver:
                    if ls[3]=='Ux,':
                        ux = float(ls[7][0:-1])
                        Ux.append(float(ls[7][0:-1]))
                    if ls[3]=='Uy,':
                        uy = float(ls[7][0:-1])
                        Uy.append(float(ls[7][0:-1]))
                        if dim==3:
                            U.append((ux,uy))
                    if ls[3]=='Uz,':
                        uz = float(ls[7][0:-1])
                        Uz.append(float(ls[7][0:-1]))
                        if dim==4:
                            U.append((ux,uy,uz))
                    if ls[3]=='k,':
                        k.append(float(ls[7][0:-1]))
                    if ls[3]=='epsilon,':
                        eps.append(float(ls[7][0:-1]))
                elif ls[0]=="ExecutionTime":
                    e.append(float(ls[2]))
            else:
                if ls[0]=="Starting":
                    started = True
    
    # Setup dictionary with the data information
    data = {}
    data['name']='pUCoupledFoam'

    pd = {}
    pd['name']='Pressure'
    pd['unit']='Pa'
    pd['data']=p

    Uxd = {}
    Uxd['name']='Velocity (x)'
    Uxd['unit']='m/s'
    Uxd['data']=Ux
    
    Uyd = {}
    Uyd['name']='Velocity (y)'
    Uyd['unit']='m/s'
    Uyd['data']=Uy

    td = {}
    td['Name']='Time'
    td['Unit']=''
    td['data']=t
    
    ed = {}
    ed['Name']='Elapsed time'
    ed['Unit']='s'
    ed['data']=e

    epsd = {}
    epsd['name']='Turbulent discipation'
    epsd['unit']=None
    epsd['data']=eps

    kd = {}
    kd['name']='Turbulent energy'
    kd['unit']=None
    kd['data']=k

    data['p'] = pd
    data['Ux'] = Uxd
    data['Uy'] = Uyd
    data['t'] = td
    data['e'] = ed
    data['eps']=epsd
    data['k']=kd

    return data

def coupledResiduals(log, dim, pUsolver):
    t = []
    i = []
    e = []
    p = []
    U = []
    Ux = []
    Uy = []
    k = []
    eps = []
    
    f = open(log)
    started = False
    for line in f:
        ls = re.sub(r'\s+',' ',line.strip()).rsplit()
        if len(ls)>0:
            if started:
                if ls[0]=="Time":
                    t.append(float(ls[2]))
                elif ls[0]=="%s:"%pUsolver:
                    if dim==2:
                        Ux.append(float(ls[7][1:]))
                        Uy.append(float(ls[8][1:]))
                        U.append((float(ls[7][1:]),float(ls[8])))
                        p.append(float(ls[9][0:-2]))
                        i.append(int(ls[-1]))         
                    if dim==3:
                        Ux.append(float(ls[7][1:]))
                        Uy.append(float(ls[8][1:]))
                        U.append((float(ls[7][1:]),float(ls[8])))
                        p.append(float(ls[10][0:-2]))
                        i.append(int(ls[-1]))         
                elif ls[0]=="DILUPBiCG:":
                    if ls[3]=='k,':
                        k.append(float(ls[7][0:-1]))
                    if ls[3]=='epsilon,':
                        eps.append(float(ls[7][0:-1]))
                if ls[0]=="ExecutionTime":
                    e.append(float(ls[2]))
            else:
                if ls[0]=="Starting":
                    started = True
    
    # Setup dictionary with the data information
    data = {}
    data['name']='pUCoupledFoam'

    pd = {}
    pd['name']='Pressure'
    pd['unit']='Pa'
    pd['data']=p

    Uxd = {}
    Uxd['name']='Velocity (x)'
    Uxd['unit']='m/s'
    Uxd['data']=Ux
    
    Uyd = {}
    Uyd['name']='Velocity (y)'
    Uyd['unit']='m/s'
    Uyd['data']=Uy

    td = {}
    td['Name']='Time'
    td['Unit']=''
    td['data']=t
    
    ed = {}
    ed['Name']='Elapsed time'
    ed['Unit']='s'
    ed['data']=e
    
    epsd = {}
    epsd['name']='Turbulent discipation'
    epsd['unit']=None
    epsd['data']=eps

    kd = {}
    kd['name']='Turbulent energy'
    kd['unit']=None
    kd['data']=k

    data['p'] = pd
    data['Ux'] = Uxd
    data['Uy'] = Uyd
    data['t'] = td
    data['e'] = ed
    data['eps']=epsd
    data['k']=kd

    return data
